"""Shared helpers for coalex.ext span decorators."""

from __future__ import annotations

from opentelemetry import trace


def _get_tracer() -> trace.Tracer:
    return trace.get_tracer("coalex.ext")
